#pragma once
#ifndef ECHO_SERVER_ASSEMBLY_MANAGER_INCLUDED__
#define ECHO_SERVER_ASSEMBLY_MANAGER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: AssemblyManager.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CLRHostingTools/HostAssemblyManager.h"
#include "JetByteTools/CLRHostingTools/CLRHost.h"

#include "JetByteTools/CoreTools/StringVector.h"
#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"
#include "JetByteTools/CoreTools/LockableObject.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Win32
   {
      class IProvideUserData;
   }

   namespace IO
   {
      class IBufferBase;
   }

   namespace Socket
   {
      class IStreamSocket;
   }

   namespace CLRHosting
   {
      class CHostMemoryManager;
   }
}

interface IManagedHost;

class CSocket;

///////////////////////////////////////////////////////////////////////////////
// CAssemblyManager
///////////////////////////////////////////////////////////////////////////////

class CAssemblyManager : public JetByteTools::CLRHosting::CHostAssemblyManager
{
   public :

      CAssemblyManager(
         const JetByteTools::Core::_tstring &assemblyStorePath,
         const JetByteTools::Core::StringVector &nonHostAssemblies);

      CAssemblyManager(
         const CAssemblyManager &rhs) = delete;

      CAssemblyManager &operator=(
         const CAssemblyManager &rhs) = delete;

   private :

      ~CAssemblyManager() override;

      // Implement IHostAssemblyStore

      HRESULT STDMETHODCALLTYPE ProvideAssembly(
         AssemblyBindInfo *pBindInfo,
         UINT64 *pAssemblyId,
         UINT64 *pContext,
         IStream **ppStmAssemblyImage,
         IStream **ppStmPDB) override;

      HRESULT STDMETHODCALLTYPE ProvideModule(
         ModuleBindInfo *pBindInfo,
         DWORD *pdwModuleId,
         IStream **ppStmModuleImage,
         IStream **ppStmPDB) override;

      JetByteTools::Core::CLockableObject m_lock;

      const JetByteTools::Core::_tstring m_assemblyStorePath;

      typedef std::pair<IStream *, __int64> AssemblyDetails;

      typedef std::map<std::wstring, AssemblyDetails> Assemblies;

      Assemblies m_assemblies;

      __int64 m_nextAssemblyId;
};

#endif // ECHO_SERVER_ASSEMBLY_MANAGER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: AssemblyManager.h
///////////////////////////////////////////////////////////////////////////////
