#pragma once
#ifndef ECHO_SERVER_MANAGED_HOST_INCLUDED__
#define ECHO_SERVER_MANAGED_HOST_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ManagedHost.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CLRHostingTools/HostControl.h"
#include "JetByteTools/CLRHostingTools/CLRHost.h"
#include "JetByteTools/CLRHostingTools/IHostPolicyManagerCallback.h"

#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"
#include "JetByteTools/CoreTools/LockableObject.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Core
   {
      class IProvideUserData;
   }

   namespace IO
   {
      class IBufferBase;
   }

   namespace Socket
   {
      class IStreamSocket;
   }

   namespace CLRHosting
   {
      class CHostPolicyManager;
   }
}

interface IManagedHost;

class CSocket;

///////////////////////////////////////////////////////////////////////////////
// CManagedHost
///////////////////////////////////////////////////////////////////////////////

class CManagedHost :
   public JetByteTools::CLRHosting::CHostControl,
   private JetByteTools::CLRHosting::IHostPolicyManagerCallback
{
   public :

      CManagedHost(
         JetByteTools::Core::IProvideUserData &dataProvider,
         const JetByteTools::Core::_tstring &welcomeMessage,
         const JetByteTools::Core::_tstring &assembly,
         const JetByteTools::Core::_tstring &type);

      ~CManagedHost();

      void Start();

      void Stop();

      void OnConnectionEstablished(
         const bool createPerConnectionApplicationDomain,
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         JetByteTools::Socket::IStreamSocket &socket);

      void OnConnectionClientClose(
         JetByteTools::Core::IIndexedOpaqueUserData &userData);

      void ReadCompleted(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const JetByteTools::IO::IBufferBase &buffer);

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData);

   private :

      IManagedHost *GetManagedHost(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) const;

      CSocket *GetManagedSocket(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) const;

      SAFEARRAY *GetManagedData(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const JetByteTools::IO::IBufferBase &buffer) const;

      void DestroyManagedData(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) const;

      IManagedHost *GetManagedHost(
         const long appDomainID) const;

      IManagedHost *GetDefaultManagedHost() const;

      // Implement IHostControl

      HRESULT STDMETHODCALLTYPE GetHostManager(
         REFIID riid,
         void **ppObject);

      HRESULT STDMETHODCALLTYPE SetAppDomainManager(
        DWORD dwAppDomainID,
        IUnknown *pUnkAppDomainManager);

      HRESULT STDMETHODCALLTYPE Setup(
         ICLRRuntimeHost *pHost);

      // Implement IHostPolicyManagerCallback

      void OnDefaultAction(
         EClrOperation operation,
         EPolicyAction action) override;

      void OnTimeout(
         EClrOperation operation,
         EPolicyAction action) override;

      void OnFailure(
         EClrFailure failure,
         EPolicyAction action) override;

      void OnException() override;

      mutable JetByteTools::Core::CLockableObject m_lock;

      JetByteTools::CLRHosting::CCLRHost *m_pClrHost;

      const JetByteTools::Core::_tstring m_welcomeMessage;

      const JetByteTools::Core::_tstring m_assembly;

      const JetByteTools::Core::_tstring m_type;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_managerIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_appDomainIDIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_managedSocketIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_managedDataIndex;

      typedef std::map<long, IManagedHost *> DomainManagerMap;

      int m_defaultHostId;

      DomainManagerMap m_domainManagers;

      ICLRControl *m_pCLRControl;

      JetByteTools::CLRHosting::CHostPolicyManager *m_pPolicyManager;
};

#endif // ECHO_SERVER_MANAGED_HOST_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ManagedHost.h
///////////////////////////////////////////////////////////////////////////////
