#pragma once
#ifndef ECHO_SERVER_SOCKET_SERVER_INCLUDED__
#define ECHO_SERVER_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include <string>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CManagedHost;

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback
{
   public :

      CSocketServer(
         CManagedHost &managedHost,
         const bool createPerConnectionApplicationDomain,
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      CSocketServer(
         const CSocketServer &rhs) = delete;

      ~CSocketServer() override;

      CSocketServer &operator=(
         const CSocketServer &rhs) = delete;

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionClientClose(
         JetByteTools::Socket::IStreamSocket &socket) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      CManagedHost &m_managedHost;

      const bool m_createPerConnectionApplicationDomain;
};

#endif // ECHO_SERVER_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
