#pragma once
#ifndef ECHO_SERVER_APPLICATION_INCLUDED__
#define ECHO_SERVER_APPLICATION_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: Application.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include <objidl.h>

#include "JetByteTools/CoreTools/LockableObject.h"
#include "JetByteTools/CoreTools/tstring.h"
#include "JetByteTools/CoreTools/ReferenceCountedSmartPointer.h"

#include <set>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Socket
   {
      class IStreamSocket;
   }
}

interface IManagedApplication;
interface ISocket;

class CApplicationCollection;

///////////////////////////////////////////////////////////////////////////////
// CApplication
///////////////////////////////////////////////////////////////////////////////

class CApplication
{
   public :

      CApplication(
         const JetByteTools::Core::_tstring &name,
         const long appDomainID,
         IManagedApplication *pApp,
         CApplicationCollection &collection);

      CApplication(
         const CApplication &rhs) = delete;

      CApplication &operator=(
         const CApplication &rhs) = delete;

      const JetByteTools::Core::_tstring &GetName() const;

      void Connect(
         JetByteTools::Socket::IStreamSocket &socket);

      void Disconnect(
         JetByteTools::Socket::IStreamSocket &socket);

      void AbortAllConnections();

      void AddRef();

      void Release();

      long GetAppDomainID() const;

      void OnConnectionEstablished(
         ISocket &socket);

      void OnConnectionClientClose(
         ISocket &socket);

      void OnReadCompleted(
         ISocket &socket,
         SAFEARRAY &data);

   private :

      ~CApplication();

      long m_ref;

      JetByteTools::Core::CLockableObject m_lock;

      const JetByteTools::Core::_tstring m_name;

      const long m_appDomainID;

      IManagedApplication *m_pApp;

      CApplicationCollection &m_collection;

      typedef std::set<JetByteTools::Socket::IStreamSocket *> Connections;

      Connections m_connections;
};

///////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////////////////////////////////////////////

typedef JetByteTools::Core::TReferenceCountedSmartPointer<CApplication> CSmartApplication;

#endif // ECHO_SERVER_APPLICATION_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: Application.h
///////////////////////////////////////////////////////////////////////////////
