#pragma once
#ifndef ECHO_SERVER_APPLICATION_COLLECTION_INCLUDED__
#define ECHO_SERVER_APPLICATION_COLLECTION_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ApplicationCollection.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/ThreadedCallbackTimerQueue.h"
#include "JetByteTools/CoreTools/LockableObject.h"

#include "JetByteTools/Win32Tools/DirectoryChangeMonitor.h"

#include "IListenForAppDomainUnload.h"
#include "ManagedHost.h"
#include "Application.h"

#include <map>
#include <deque>

///////////////////////////////////////////////////////////////////////////////
// CApplicationCollection
///////////////////////////////////////////////////////////////////////////////

class CApplicationCollection :
   private IListenForAppDomainUnload,
   private JetByteTools::Core::IQueueTimers::Timer
{
   public :

      enum AssemblyReloadFlags
      {
         DisableShadowCopy,
         EnableShadowCopy,
         EnableAutoRestart,
         EnableAutoForceRestart
      };

      class ApplicationDetails : private JetByteTools::Win32::CDirectoryChangeMonitor::Callback
      {
         public :

            ApplicationDetails(
               JetByteTools::Win32::CDirectoryChangeMonitor &monitor,
               JetByteTools::Core::CThreadedCallbackTimerQueue &timerQueue,
               JetByteTools::Core::IQueueTimers::Timer &timer,
               const JetByteTools::Core::_tstring &baseDirectory,
               const JetByteTools::Core::_tstring &application,
               const JetByteTools::Core::_tstring &applicationDirectory,
               const JetByteTools::Core::_tstring &sharedDirectory,
               const JetByteTools::Core::_tstring &configFile,
               const AssemblyReloadFlags assemblyReloadFlags,
               const JetByteTools::Core::_tstring &assembly,
               const JetByteTools::Core::_tstring &type);

            ApplicationDetails(
               const ApplicationDetails &rhs) = delete;

            ApplicationDetails &operator=(
               const ApplicationDetails &rhs) = delete;


            void AddRef();

            void Release();

            JetByteTools::Core::_tstring application;
            JetByteTools::Core::_tstring applicationDirectory;
            JetByteTools::Core::_tstring sharedDirectory;
            JetByteTools::Core::_tstring configFile;
            AssemblyReloadFlags assemblyReloadFlags;
            JetByteTools::Core::_tstring assembly;
            JetByteTools::Core::_tstring type;

         private :

            ~ApplicationDetails() override;

            // Implement CDirectoryChangeMonitor::Callback

            void OnChange(
               const JetByteTools::Core::_tstring &directoryName,
               const DWORD action,
               const JetByteTools::Core::_tstring &fileName,
               const DWORD userData) override;

            friend class CApplicationCollection;

            void BeginMonitoring();

            void EndMonitoring();

            JetByteTools::Win32::CDirectoryChangeMonitor &m_monitor;

            JetByteTools::Core::CThreadedCallbackTimerQueue &m_timerQueue;

            JetByteTools::Core::IQueueTimers::Handle m_timerHandle;

            JetByteTools::Core::IQueueTimers::Timer &m_timer;

            const JetByteTools::Core::_tstring m_baseDirectory;

            volatile long m_ref;

            typedef std::deque<JetByteTools::Win32::CDirectoryChangeMonitor::Handle> DirectoryChangeHandles;

            DirectoryChangeHandles m_handles;
      };

      typedef std::map<JetByteTools::Core::_tstring, ApplicationDetails *> ApplicationDetailsMap;

      typedef ApplicationDetailsMap::const_iterator Iterator;

      CApplicationCollection(
         const JetByteTools::Core::_tstring &baseDirectory,
         const JetByteTools::Core::_tstring &assembly,
         const JetByteTools::Core::_tstring &type,
         const bool enableManagedDebuggingAssistants = false);

      CApplicationCollection(
         const CApplicationCollection &rhs) = delete;

      ~CApplicationCollection() override;

      CApplicationCollection &operator=(
         const CApplicationCollection &rhs) = delete;

      void AddApplication(
         const JetByteTools::Core::_tstring &application,
         const JetByteTools::Core::_tstring &applicationDirectory,
         const JetByteTools::Core::_tstring &sharedDirectory,
         const JetByteTools::Core::_tstring &configFile,
         const AssemblyReloadFlags assemblyReloadFlags,
         const JetByteTools::Core::_tstring &assembly,
         const JetByteTools::Core::_tstring &type);

      bool StartApplication(
         const JetByteTools::Core::_tstring &application);

      CSmartApplication GetApplication(
         const JetByteTools::Core::_tstring &application) const;

      bool RestartApplication(
         const JetByteTools::Core::_tstring &application,
         const bool kill = false);

      bool StopApplication(
         const JetByteTools::Core::_tstring &application,
         const bool kill = false);

      void StopAllApplications(
         const bool kill = false);

      bool ApplicationIsRunning(
         const JetByteTools::Core::_tstring &application) const;

      void RemoveApplication(
         const JetByteTools::Core::_tstring &application,
         const bool kill = false);

      void RemoveAllApplications(
         const bool kill = false);

      Iterator Begin() const;

      Iterator End() const;

   private :

      // Implement IListenForAppDomainUnload

      void OnAppDomainUnload(
         const long appDomainID) override;

      // Implement IQueueTimers::Timer

      void OnTimer(
         UserData userData) override;

      friend class CApplication;

      void OnApplicationDestroyed(
         const long appDomainID);

      ApplicationDetails &GetApplicationDetails(
         const JetByteTools::Core::_tstring &application);

      const ApplicationDetails &GetApplicationDetails(
         const JetByteTools::Core::_tstring &application) const;

      void ValidateApplication(
         const JetByteTools::Core::_tstring &application) const;

      void StopApplication(
         CApplication &application,
         const bool kill);

      mutable JetByteTools::Core::CLockableObject m_lock;

      typedef std::map<JetByteTools::Core::_tstring, CApplication *> ApplicationByNameMap;

      ApplicationByNameMap m_applicationsByName;

      typedef std::map<int, CApplication *> ApplicationByIdMap;

      ApplicationByIdMap m_applicationsById;

      ApplicationDetailsMap m_applicationDetails;

      const JetByteTools::Core::_tstring m_baseDirectory;

      CManagedHost m_host;

      JetByteTools::Win32::CDirectoryChangeMonitor m_directoryChangeMonitor;

      JetByteTools::Core::CThreadedCallbackTimerQueue m_timerQueue;
};

#endif // ECHO_SERVER_APPLICATION_COLLECTION_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ApplicationCollection.h
///////////////////////////////////////////////////////////////////////////////
