#pragma once
#ifndef ECHO_SERVER_CONFIGURATION_SOCKET_SERVER_INCLUDED__
#define ECHO_SERVER_CONFIGURATION_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ConfigurationSocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServer.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

#include "JetByteTools/CoreTools/IndexedOpaqueUserData.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CApplicationCollection;

///////////////////////////////////////////////////////////////////////////////
// CConfigurationSocketServer
///////////////////////////////////////////////////////////////////////////////

class CConfigurationSocketServer :
   public JetByteTools::Socket::CStreamSocketServer,
   private CStreamSocketServerExCallback
{
   public :

      CConfigurationSocketServer(
         CApplicationCollection &applications,
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      CConfigurationSocketServer(
         const CConfigurationSocketServer &rhs) = delete;

      ~CConfigurationSocketServer() override;

      CConfigurationSocketServer &operator=(
         const CConfigurationSocketServer &rhs) = delete;

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void ProcessDataStream(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      JetByteTools::IO::IBuffer::BufferSize GetMinimumMessageSize() const;

      JetByteTools::IO::IBuffer::BufferSize GetMessageSize(
         const JetByteTools::IO::IBuffer &buffer,
         const JetByteTools::IO::IBuffer::BufferSize startOffset) const;

      void ProcessCommand(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::IO::CSmartBuffer &buffer);

      void ShowHelp(
         JetByteTools::Socket::IStreamSocket &socket) const;

      void ListApplications(
         JetByteTools::Socket::IStreamSocket &socket) const;

      bool StopApplication(
         const std::string &command,
         JetByteTools::Socket::IStreamSocket &socket,
         const bool kill);

      bool StartApplication(
         const std::string &command,
         JetByteTools::Socket::IStreamSocket &socket);

      bool RestartApplication(
         const std::string &command,
         JetByteTools::Socket::IStreamSocket &socket,
         const bool kill);

      CApplicationCollection &m_applications;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;
};

#endif // ECHO_SERVER_CONFIGURATION_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ConfigurationSocketServer.h
///////////////////////////////////////////////////////////////////////////////
