#pragma once
#ifndef ECHO_SERVER_MANAGED_HOST_INCLUDED__
#define ECHO_SERVER_MANAGED_HOST_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ManagedHost.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "ManagedHostInterface/IManagedHost.h"

#include "JetByteTools/CLRHostingTools/HostControl.h"
#include "JetByteTools/CLRHostingTools/CLRHost.h"
#include "JetByteTools/CLRHostingTools/IHostPolicyManagerCallback.h"
#include "JetByteTools/CLRHostingTools/IEventSinkCallback.h"
#include "JetByteTools/CLRHostingTools/HostPolicyManager.h"

#include "JetByteTools/CoreTools/LockableObject.h"
#include "JetByteTools/CoreTools/ReferenceCountedSmartPointer.h"

#include <map>
#include <memory>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class IListenForAppDomainUnload;

///////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////////////////////////////////////////////

typedef JetByteTools::Core::TReferenceCountedSmartPointer<IManagedHost> CSmartManagedHost;

///////////////////////////////////////////////////////////////////////////////
// CManagedHost
///////////////////////////////////////////////////////////////////////////////

class CManagedHost :
   public JetByteTools::CLRHosting::CHostControl,
   private JetByteTools::CLRHosting::IHostPolicyManagerCallback,
   private JetByteTools::CLRHosting::IEventSinkCallback
{
   public :

      CManagedHost(
         const JetByteTools::Core::_tstring &assembly,
         const JetByteTools::Core::_tstring &type,
         IListenForAppDomainUnload &domainUnloadListener,
         const bool enableManagedDebuggingAssistants = false);

      CManagedHost(
         const CManagedHost &rhs) = delete;

      ~CManagedHost() override;

      CManagedHost &operator=(
         const CManagedHost &rhs) = delete;

      long CreateApplicationInNewAppDomain(
         const JetByteTools::Core::_tstring &application,
         const JetByteTools::Core::_tstring &applicationDirectory,
         const JetByteTools::Core::_tstring &sharedDirectory,
         const JetByteTools::Core::_tstring &configFile,
         const bool enableShadowCopy);

      IManagedApplication *StartApplication(
         const long appDomainID,
         const JetByteTools::Core::_tstring &assembly,
         const JetByteTools::Core::_tstring &type,
         const JetByteTools::Core::_tstring &application);

      void RequestApplicationStop(
         const long appDomainID) const;

      void StopApplication(
         const long appDomainID) const;

      void DestroyAppDomain(
         const long appDomainID);

   private :

      void Start(
         const bool enableManagedDebuggingAssistants);

      void SetupCLRErrorEscalationPolicy();

      void ConnectDefaultAppDomainManager();

      void Stop();

      CSmartManagedHost GetManagedHost(
         const long appDomainID) const;

      CSmartManagedHost GetDefaultManagedHost() const;

      // Implement IHostControl

      HRESULT STDMETHODCALLTYPE GetHostManager(
         REFIID riid,
         void **ppObject);

      HRESULT STDMETHODCALLTYPE SetAppDomainManager(
        DWORD dwAppDomainID,
        IUnknown *pUnkAppDomainManager);

      HRESULT STDMETHODCALLTYPE Setup(
         ICLRRuntimeHost *pHost);

      // Implement IHostPolicyManagerCallback

      void OnDefaultAction(
         EClrOperation operation,
         EPolicyAction action) override;

      void OnTimeout(
         EClrOperation operation,
         EPolicyAction action) override;

      void OnFailure(
         EClrFailure failure,
         EPolicyAction action) override;

      // Implement IEventSinkCallback

      void OnCLRDisabled() override;

      void OnDomainUnload(
         const int domainId) override;

      void OnMDAEvent(
         const JetByteTools::Core::_tstring &title,
         const JetByteTools::Core::_tstring &message) override;

      void OnStackOverflow() override;

      void OnException() override;

      std::unique_ptr<JetByteTools::CLRHosting::CCLRHost> m_spClrHost;

      const JetByteTools::Core::_tstring m_assembly;

      const JetByteTools::Core::_tstring m_type;

      IListenForAppDomainUnload &m_domainUnloadListener;

      mutable JetByteTools::Core::CLockableObject m_lock;

      typedef std::map<long, IManagedHost *> DomainManagerMap;

      int m_defaultHostId;

      DomainManagerMap m_domainManagers;

      typedef JetByteTools::Core::TReferenceCountedSmartPointer<ICLRControl> CSmartCLRControl;

      CSmartCLRControl m_spCLRControl;

      typedef JetByteTools::Core::TReferenceCountedSmartPointer<IUnmanagedHost> CSmartUnmanagedHost;

      CSmartUnmanagedHost m_spUnmanagedHost;

      typedef JetByteTools::Core::TReferenceCountedSmartPointer<JetByteTools::CLRHosting::CHostPolicyManager> CSmartHostPolicyManager;

      CSmartHostPolicyManager m_spPolicyManager;
};

#endif // ECHO_SERVER_MANAGED_HOST_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ManagedHost.h
///////////////////////////////////////////////////////////////////////////////
