///////////////////////////////////////////////////////////////////////////////
// File: PerConnectionData.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"
#include "JetByteTools/Admin/WinsockWrapper.h"

#include "PerConnectionData.h"
#include "Application.h"
#include "Socket.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/IReentrantLockableObject.h"

#include "JetByteTools/IOTools/IBuffer.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#include "JetByteTools/CLRHostingTools/Exception.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::IReentrantLockableObject;

using JetByteTools::IO::IBuffer;

using JetByteTools::Socket::IStreamSocket;

///////////////////////////////////////////////////////////////////////////////
// Static helper functions
///////////////////////////////////////////////////////////////////////////////

static SAFEARRAY *CreateSafeArray();

///////////////////////////////////////////////////////////////////////////////
// CPerConnectionData
///////////////////////////////////////////////////////////////////////////////

CPerConnectionData::CPerConnectionData(
   CApplication &application,
   IStreamSocket &socket)
   :  m_lock(socket.GetLock()),
      m_application(application),
      m_pSocket(new CSocket(socket)),
      m_pArray(CreateSafeArray())
{
   m_application.Connect(socket);
}

CPerConnectionData::~CPerConnectionData()
{
   try
   {
      SafeArrayDestroyDescriptor(m_pArray);

      IStreamSocket &socket = m_pSocket->GetSocket();

      m_pSocket->Disconnect();

      m_application.Disconnect(socket);
   }
   JETBYTE_CATCH_AND_LOG_ALL_IN_DESTRUCTORS_IF_ENABLED
}

void CPerConnectionData::OnConnectionEstablished()
{
   m_application.OnConnectionEstablished(*m_pSocket);
}

void CPerConnectionData::OnReadCompleted(
   const IBuffer &buffer)
{
   IReentrantLockableObject::Owner lock(m_lock);

   m_pArray->pvData = const_cast<BYTE *>(buffer.GetMemory());
   m_pArray->rgsabound[0].cElements = buffer.GetUsed();

   m_application.OnReadCompleted(*m_pSocket, *m_pArray);
}

void CPerConnectionData::OnConnectionClientClose()
{
   m_application.OnConnectionClientClose(*m_pSocket);
}

///////////////////////////////////////////////////////////////////////////////
// Static helper functions
///////////////////////////////////////////////////////////////////////////////

static SAFEARRAY *CreateSafeArray()
{
   SAFEARRAY *pArray = 0;

   HRESULT hr = SafeArrayAllocDescriptorEx(VT_UI1, 1, &pArray);

   JetByteTools::CLRHosting::CException::ThrowOnFailure(_T("CPerConnectionData::CreateSafeArray() - SafeArrayAllocDescriptorEx"), hr);

   pArray->pvData = 0;
   pArray->rgsabound[0].lLbound = 0;
   pArray->rgsabound[0].cElements = 0;

   return pArray;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: PerConnectionData.cpp
///////////////////////////////////////////////////////////////////////////////
