///////////////////////////////////////////////////////////////////////////////
// File: Socket.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"
#include "JetByteTools/Admin/WinsockWrapper.h"

#include "Socket.h"

#include "JetByteTools/CoreTools/ToString.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/StringConverter.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop


///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::DumpData;
using JetByteTools::Core::CStringConverter;
using JetByteTools::Core::GetStringLength;
using JetByteTools::Core::CException;

using JetByteTools::Socket::IStreamSocket;

///////////////////////////////////////////////////////////////////////////////
// CSocket
///////////////////////////////////////////////////////////////////////////////

CSocket::CSocket(
   IStreamSocket &socket)
   :  m_pSocket(&socket),
      m_ref(1)
{

}

CSocket::~CSocket()
{

}

IStreamSocket &CSocket::GetSocket() const
{
   if (!m_pSocket)
   {
      throw CException(_T("CSocket::GetSocket()"), _T("Too late, socket is null"));
   }

   return *m_pSocket;
}

void CSocket::Disconnect()
{
   m_pSocket = 0;

   Release();
}

ULONG CSocket::AddRef()
{
   return ::InterlockedIncrement(&m_ref);
}

ULONG CSocket::Release()
{
   ULONG ref = ::InterlockedDecrement(&m_ref);

   if (ref == 0)
   {
      delete this;
   }

   return ref;
}

HRESULT CSocket::QueryInterface(
   REFIID riid,
   PVOID *ppvObj)
{
   HRESULT hr = S_OK;

   if (riid == IID_IUnknown)
   {
      *ppvObj = static_cast<IUnknown *>(this);
   }
   else if (riid == IID_ISocket)
   {
      *ppvObj = static_cast<ISocket*>(this);
   }
   else
   {
      hr = E_NOINTERFACE;
      *ppvObj = NULL;
   }

   if (hr == S_OK)
   {
      AddRef();
   }

   return hr;
}

// Implement ISocket

HRESULT CSocket::WriteString(
  BSTR data)
{
   if (m_pSocket)
   {
      const std::string message = CStringConverter::BSTRtoA(data);

      if (m_pSocket->TryWrite(message.c_str(), GetStringLength<DWORD>(message)))
      {
         return S_OK;
      }
   }

   return E_FAIL;
}

HRESULT CSocket::Write(
   SAFEARRAY *pData)
{
   if (m_pSocket)
   {
      if (m_pSocket->TryWrite(reinterpret_cast<BYTE *>(pData->pvData), pData->rgsabound[0].cElements))
      {
         DEBUG_ONLY(Output(_T("\r\n") + DumpData(reinterpret_cast<BYTE *>(pData->pvData), pData->rgsabound[0].cElements, 60, true)));

         return S_OK;
      }
   }

   return E_FAIL;
}

HRESULT CSocket::Read()
{
   if (m_pSocket)
   {
      if (m_pSocket->TryRead())
      {
         return S_OK;
      }
   }

   return E_FAIL;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: Socket.cpp
///////////////////////////////////////////////////////////////////////////////
