///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "SocketServer.h"
#include "ApplicationCollection.h"
#include "Application.h"
#include "PerConnectionData.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/Exception.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop

#include "JetByteTools/CoreTools/StringConverter.h"

#include "JetByteTools/SocketTools/IAllocateStreamSockets.h"

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;
using JetByteTools::Core::Output;
using JetByteTools::Core::CException;
using JetByteTools::Core::CStringConverter;
using JetByteTools::Core::IIndexedOpaqueUserData;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::CSmartBuffer;
using JetByteTools::IO::IIOPool;

using JetByteTools::Socket::CStreamSocketServerEx;
using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::IAllocateStreamSockets;
using JetByteTools::Socket::IFullAddress;
using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::ILimitConnections;
using JetByteTools::Socket::ListenBacklog;
using JetByteTools::Socket::NoZeroByteRead;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

CSocketServer::CSocketServer(
   CApplicationCollection &applications,
   const _tstring &applicationName,
   const IFullAddress &address,
   const ListenBacklog listenBacklog,
   IIOPool &pool,
   IAllocateStreamSockets &socketAllocator,
   IAllocateBuffers &bufferAllocator,
   ILimitConnections &connectionLimiter)
   :  CStreamSocketServerEx(address, listenBacklog, *this, pool, socketAllocator, bufferAllocator, NoZeroByteRead, connectionLimiter),
      m_applications(applications),
      m_applicationName(applicationName),
      m_userDataIndex(socketAllocator.RequestUserDataSlot(_T("CSocketServer - m_userDataIndex")))
{

}

CSocketServer::~CSocketServer()
{
   try
   {
      WaitForShutdownToComplete();
   }
   JETBYTE_CATCH_AND_LOG_ALL_IN_DESTRUCTORS_IF_ENABLED
}

void CSocketServer::OnConnectionEstablished(
   IStreamSocket &socket,
   const IAddress & /*address*/)
{
   Output(_T("OnConnectionEstablished"));

   CSmartApplication spApplication(m_applications.GetApplication(m_applicationName));

   if (spApplication.Get())
   {
      CPerConnectionData *pData = new CPerConnectionData(spApplication.GetRef(), socket);

      socket.SetUserPointer(m_userDataIndex, pData);

      pData->OnConnectionEstablished();
   }
   else
   {
      const string message = "Application: \"" + CStringConverter::TtoA(m_applicationName) + "\" not available at this time.";

      socket.Write(message.c_str(), static_cast<DWORD>(message.length()));

      socket.Shutdown();
   }
}

void CSocketServer::OnConnectionClientClose(
   IStreamSocket &socket)
{
   Output(_T("OnConnectionClientClose"));

   CPerConnectionData *pData = reinterpret_cast<CPerConnectionData *>(socket.GetUserPointer(m_userDataIndex));

   pData->OnConnectionClientClose();
}

void CSocketServer::OnReadCompleted(
   IStreamSocket &socket,
   CSmartBuffer &buffer)
{
   Output(_T("OnReadCompleted"));

   CPerConnectionData *pData = reinterpret_cast<CPerConnectionData *>(socket.GetUserPointer(m_userDataIndex));

   pData->OnReadCompleted(buffer.GetRef());

   buffer->Empty();
}

void CSocketServer::OnSocketReleased(
   IIndexedOpaqueUserData &userData)
{
   CPerConnectionData *pData = reinterpret_cast<CPerConnectionData *>(userData.GetUserPointer(m_userDataIndex));

   delete pData;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
