///////////////////////////////////////////////////////////////////////////////
// File: UnmanagedHost.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "UnmanagedHost.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/StringConverter.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::OutputEx;
using JetByteTools::Core::CStringConverter;

///////////////////////////////////////////////////////////////////////////////
// CUnmanagedHost
///////////////////////////////////////////////////////////////////////////////

CUnmanagedHost::CUnmanagedHost()
   :  m_ref(1)
{

}

CUnmanagedHost::~CUnmanagedHost()
{

}

ULONG CUnmanagedHost::AddRef()
{
   return ::InterlockedIncrement(&m_ref);
}

ULONG CUnmanagedHost::Release()
{
   ULONG ref = ::InterlockedDecrement(&m_ref);

   if (ref == 0)
   {
      delete this;
   }

   return ref;
}

HRESULT CUnmanagedHost::QueryInterface(
   REFIID riid,
   PVOID *ppvObj)
{
   HRESULT hr = S_OK;

   if (riid == IID_IUnknown)
   {
      *ppvObj = static_cast<IUnknown *>(this);
   }
   else if (riid == IID_IUnmanagedHost)
   {
      *ppvObj = static_cast<IUnmanagedHost*>(this);
   }
   else if (riid == IID_IHostLog)
   {
      *ppvObj = static_cast<IHostLog*>(this);
   }
   else
   {
      hr = E_NOINTERFACE;
      *ppvObj = NULL;
   }

   if (hr == S_OK)
   {
      AddRef();
   }

   return hr;
}

// Implement IUnmanagedHost

HRESULT CUnmanagedHost::ShutdownHost()
{
   return E_NOTIMPL;
}

HRESULT CUnmanagedHost::LogMessage(
  BSTR data)
{
   OutputEx(CStringConverter::BSTRtoT(data));

   return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: UnmanagedHost.cpp
///////////////////////////////////////////////////////////////////////////////
