@Echo off

REM Most arguments passed to us via the environment as they contain spaces...

if not defined SERVER goto NotSet
if not defined BASE_ARGS goto NotSet
if not defined OUT_DIR goto NotSet

goto DisplayDebug

:NotSet
Echo One or more of the following variables is not set:
Echo -
Echo SERVER=%SERVER%
Echo BASE_ARGS=%BASE_ARGS%
Echo OUT_DIR=%OUT_DIR%
Echo -
Echo Please ensure they are all set in the environment before calling.
Exit /b 1

:DisplayDebug

if not defined JETBYTE_DEBUG_BUILD_SCRIPTS goto NoDebug

Echo Script debug information - RunInstrumentedTests.bat
Echo - 
Echo Required: SERVER=%SERVER%
Echo Required: BASE_ARGS=%BASE_ARGS%
Echo Required: OUT_DIR=%OUT_DIR%
Echo Optional: TEST_SCRIPT=%TEST_SCRIPT%
Echo Optional: MEMORY_LIMIT=%MEMORY_LIMIT%
Echo Optional: SERVER_USE_LID=%SERVER_USE_LID%
Echo Optional: SERVER_USE_LIA=%SERVER_USE_LIA%
Echo Optional: CLIENT_USE_LID=%CLIENT_USE_LID%
Echo Optional: CLIENT_USE_LIA=%CLIENT_USE_LIA%
Echo Optional: CHANGE_TO_WORKING_DIR=%CHANGE_TO_WORKING_DIR%
Echo -
Echo PROCESSOR_ARCHITECTURE=%PROCESSOR_ARCHITECTURE%
Echo PROCESSOR_ARCHITEW6432=%PROCESSOR_ARCHITEW6432%
Echo -

:NoDebug

set BASE_TEST_NAME=
set ORIG_BASE_TEST_NAME=

if not defined TEST_SCRIPT set TEST_SCRIPT=..\..\Test\RunTests.bat

if defined PROCESSOR_ARCHITEW6432 goto Check6432

if %PROCESSOR_ARCHITECTURE%==x86 goto RunInstrumentedTests

if %PROCESSOR_ARCHITECTURE%==AMD64 goto RunInstrumentedTests64

Echo Instrumented tests are currently only supported on x86 and AMD64

goto RunTests

:Check6432

if %PROCESSOR_ARCHITEW6432%==AMD64 goto RunInstrumentedTests64

goto RunTests

Echo

:RunInstrumentedTests64

set LID_NAME_SUFFIX=64
set LIA_NAME_SUFFIX=64

:RunInstrumentedTests

if defined CHANGE_TO_WORKING_DIR set TEST_RUNNER_ADDITIONAL_ARGS=-dir %CHANGE_TO_WORKING_DIR%

if not defined CLIENT_USE_LID goto ClientWithoutLID

REM We currently assume that the client's are ALWAYS x86

set TST_RUNNER=..\..\Test\LID.exe
REM set TST_RUNNER=..\..\Test\LID%LID_NAME_SUFFIX%.exe

copy %TST_RUNNER% NUL > NUL
if not ERRORLEVEL 1 goto RunLIDOnClient

Echo ***************************************************************
Echo * Tool not available: Skipping deadlock detection tests       *
Echo * Note: LID.exe is available from http://www.lockexplorer.com *
Echo ***************************************************************

goto ClientConfigured

:RunLIDOnClient

set CLIENT_TEST_RUNNER=%TST_RUNNER%
set CLIENT_TEST_RUNNER_ARGS=-skipSingleThread -captureOut Client.output.log -captureErr Client.err.log %TEST_RUNNER_ADDITIONAL_ARGS%
set ORIG_BASE_TEST_NAME=Client LID -

goto ClientConfigured

:ClientWithoutLID

if not defined CLIENT_USE_LIA goto ClientConfigured

REM We currently assume that the client's are ALWAYS x86

set TST_RUNNER=..\..\Test\LIA.exe
REM set TST_RUNNER=..\..\Test\LIA%LIA_NAME_SUFFIX%.exe

copy %TST_RUNNER% NUL > NUL
if not ERRORLEVEL 1 goto RunLIAOnClient

Echo ***************************************************************
Echo * Tool not available: Skipping deadlock detection tests       *
Echo * Note: LIA.exe is available from http://www.lockexplorer.com *
Echo ***************************************************************

goto ClientConfigured

:RunLIAOnClient

set CLIENT_TEST_RUNNER=%TST_RUNNER%
set CLIENT_TEST_RUNNER_ARGS=-skipSingleThread %TEST_RUNNER_ADDITIONAL_ARGS%
set ORIG_BASE_TEST_NAME=Client LIA -

:ClientConfigured

if not defined MEMORY_LIMIT goto UseLID

Echo ******************************
Echo * Running memory usage tests *
Echo ******************************

:CheckMemUsageToolAvail

set TEST_RUNNER=..\..\Test\TestRunner.exe

copy %TEST_RUNNER% NUL > NUL
if not ERRORLEVEL 1 goto RunMemUsage

Echo ***************************************************
Echo * Tool not available: Skipping memory usage tests *
Echo ***************************************************

goto UseLID

:RunMemUsage

set TEST_RUNNER_ID_FILE=%OUT_DIR%\TestRunner-ID.txt
set TEST_RUNNER_ARGS=-logName TestRunner-Server.log -memoryLimit %MEMORY_LIMIT% -showStats -terminateOnLimit -useID %TEST_RUNNER_ID_FILE% %TEST_RUNNER_ADDITIONAL_ARGS%
set TEST_RUNNER_ID_GENERATOR=%TEST_RUNNER% -generateID %TEST_RUNNER_ID_FILE%
set TEST_RUNNER_WAITER=%TEST_RUNNER% -waitID %TEST_RUNNER_ID_FILE%
set BASE_TEST_NAME=%ORIG_BASE_TEST_NAME% MemUsage -

if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Calling: TEST_SCRIPT (%TEST_SCRIPT%)
call %TEST_SCRIPT%
if ERRORLEVEL 1 Exit /b %ERRORLEVEL%
if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Back from: TEST_SCRIPT (%TEST_SCRIPT%)

goto UseLID

:RunMemUsageNotSupported

:UseLID

if not defined SERVER_USE_LID goto UseLIA

Echo ******************************************
Echo * Running lock inversion detection tests *
Echo ******************************************

set TEST_RUNNER=..\..\Test\LID%LID_NAME_SUFFIX%.exe

copy %TEST_RUNNER% NUL > NUL
if not ERRORLEVEL 1 goto RunLID

Echo ***************************************************************
Echo * Tool not available: Skipping lock inversion detection tests *
Echo * Note: LID.exe is available from http://www.lockexplorer.com *
Echo ***************************************************************

goto UseLIA

:RunLID

set TEST_RUNNER_ID_FILE=%OUT_DIR%\LID-ID.txt
set TEST_RUNNER_ARGS=-skipSingleThread -useID %TEST_RUNNER_ID_FILE% %TEST_RUNNER_ADDITIONAL_ARGS%
set TEST_RUNNER_ID_GENERATOR=%TEST_RUNNER% -generateID %TEST_RUNNER_ID_FILE% -out LID-GenerateID.log
set TEST_RUNNER_WAITER=%TEST_RUNNER% -waitID %TEST_RUNNER_ID_FILE% -out LID-WaitID.log
set BASE_TEST_NAME=%ORIG_BASE_TEST_NAME% Server LID -

REM Run the test...

if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Calling: TEST_SCRIPT (%TEST_SCRIPT%)
call %TEST_SCRIPT%
if ERRORLEVEL 1 Exit /b %ERRORLEVEL%
if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Back from: TEST_SCRIPT (%TEST_SCRIPT%)

:UseLIA

if not defined SERVER_USE_LIA goto RunTests

Echo ************************************
Echo * Running deadlock detection tests *
Echo ************************************

set TEST_RUNNER=..\..\Test\LIA%LIA_NAME_SUFFIX%.exe

copy %TEST_RUNNER% NUL > NUL
if not ERRORLEVEL 1 goto RunDeadlock

Echo ***************************************************************
Echo * Tool not available: Skipping deadlock detection tests       *
Echo * Note: LIA.exe is available from http://www.lockexplorer.com *
Echo ***************************************************************

goto RunTests

:RunDeadlock

set TEST_RUNNER_ID_FILE=%OUT_DIR%\LIA-ID.txt
set TEST_RUNNER_ARGS=-skipSingleThread -useID %TEST_RUNNER_ID_FILE% %TEST_RUNNER_ADDITIONAL_ARGS%
set TEST_RUNNER_ID_GENERATOR=%TEST_RUNNER% -generateID %TEST_RUNNER_ID_FILE% -out LIA-GenerateID.log
set TEST_RUNNER_WAITER=%TEST_RUNNER% -waitID %TEST_RUNNER_ID_FILE% -out LIA-WaitID.log
set BASE_TEST_NAME=%ORIG_BASE_TEST_NAME% Server LIA -

REM set TEST_RUNNER_ID_FILE=%OUT_DIR%\LEX-ID.txt
REM set TEST_RUNNER=..\..\Test\LockExplorerExporter.exe
REM set TEST_RUNNER_ARGS=-skipSingleThread -useID %TEST_RUNNER_ID_FILE% %TEST_RUNNER_ADDITIONAL_ARGS%
REM set TEST_RUNNER_ID_GENERATOR=%TEST_RUNNER% -generateID %TEST_RUNNER_ID_FILE% -out LEX-GenerateID.log
REM set TEST_RUNNER_WAITER=%TEST_RUNNER% -waitID %TEST_RUNNER_ID_FILE% -out LEX-WaitID.log

REM Run the test...

if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Calling: TEST_SCRIPT (%TEST_SCRIPT%)
call %TEST_SCRIPT%
if ERRORLEVEL 1 Exit /b %ERRORLEVEL%
if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Back from: TEST_SCRIPT (%TEST_SCRIPT%)

:RunTests

set TEST_RUNNER_ID_FILE=
set TEST_RUNNER=
set TEST_RUNNER_ARGS=
set TEST_RUNNER_ID_GENERATOR=
set TEST_RUNNER_WAITER=
set CLIENT_TEST_RUNNER=
set CLIENT_TEST_RUNNER_ARGS=
set ORIG_BASE_TEST_NAME=
set BASE_TEST_NAME=%ORIG_BASE_TEST_NAME%

if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Calling: TEST_SCRIPT (%TEST_SCRIPT%)
call %TEST_SCRIPT%
if ERRORLEVEL 1 Exit /b %ERRORLEVEL%
if defined JETBYTE_DEBUG_BUILD_SCRIPTS Echo Back from: TEST_SCRIPT (%TEST_SCRIPT%)

:Done

